from collections import OrderedDict
from czsc.analyze import CZSC
from czsc.signals.tas import update_ma_cache
from czsc.utils import create_single_signal


def tas_dma_bs_V240608(c: CZSC, **kwargs) -> OrderedDict:
    """双均线多头排列下的回调买点

    参数模板："{freq}_N{n}双均线{t1}#{t2}顺势_BS辅助V240608"

    **信号逻辑：**

    参考链接：https://mp.weixin.qq.com/s/hR6wl3UrWvmLm1j5EABVyA

    买点的定位以均线为主，要求如下。
    1，做多的情况下5日均线和10日均线必须多头排列，做空的情况下5日均线和10日均线必须空头排列。
    2，以做多为例，做空反过来就是：日线价格回调到到5日均线或者10日均线。

    **信号列表：**

    - Signal('60分钟_N5双均线5#13顺势_BS辅助V240608_买点_任意_任意_0')
    - Signal('60分钟_N5双均线5#13顺势_BS辅助V240608_卖点_任意_任意_0')

    :param c: CZSC对象
    :param kwargs: 无

        - n: int, 默认5，取最近均线附近n个价格
        - t1: int, 默认5，均线1的周期
        - t2: int, 默认10，均线2的周期
    :return: 信号识别结果
    """
    n = int(kwargs.get('n', 5))
    t1 = int(kwargs.get('t1', 5))
    t2 = int(kwargs.get('t2', 10))

    assert t1 < t2, "均线1的周期必须小于均线2的周期"

    freq = c.freq.value
    k1, k2, k3 = f"{freq}_N{n}双均线{t1}#{t2}顺势_BS辅助V240608".split('_')
    v1 = '其他'
    ma1 = update_ma_cache(c, timeperiod=t1)
    ma2 = update_ma_cache(c, timeperiod=t2)
    if len(c.bars_raw) < 110:
        return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)

    bars = c.bars_raw[-100:]
    unique_prices = [x.close for x in bars] + [x.high for x in bars] + [x.low for x in bars] + [x.open for x in bars]
    unique_prices = sorted(list(set(unique_prices)))

    bar1, bar2 = bars[-2], bars[-1]
    ma1_value, ma2_value = bar2.cache[ma1], bar2.cache[ma2]
    lower_prices = [x for x in unique_prices if x < ma2_value]
    upper_prices = [x for x in unique_prices if x > ma2_value]

    if upper_prices and ma1_value > ma2_value and bar2.cache[ma2] > bar1.cache[ma2]:
        # ma2_round_high 是 ma2_value 上方的第 n 个价格
        ma2_round_high = upper_prices[n] if len(upper_prices) > n else upper_prices[-1]
        # 买点：1）上一根K线的最低价小于 ma2_round_high；2）当前K线的最高价大于 ma2_round_high，且收盘价小于 ma2_round_high
        if bar1.low < ma2_round_high < bar2.high and bar2.close < ma2_round_high:
            v1 = '买点'

    elif lower_prices and ma1_value < ma2_value and bar2.cache[ma2] < bar1.cache[ma2]:
        # ma2_round_low 是 ma2_value 下方的第 n 个价格
        ma2_round_low = lower_prices[-n] if len(lower_prices) > n else lower_prices[0]
        # 卖点：1）上一根K线的最高价大于 ma2_round_low；2）当前K线的收盘价大于 ma2_round_low，且收盘价大于 ma2_round_low
        if bar1.high > ma2_round_low > bar2.low and bar2.close > ma2_round_low:
            v1 = '卖点'

    return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)


def check():
    from czsc.connectors import research
    from czsc.traders.base import check_signals_acc

    symbols = research.get_symbols('A股主要指数')
    bars = research.get_raw_bars(symbols[0], '15分钟', '20181101', '20210101', fq='前复权')

    signals_config = [{'name': tas_dma_bs_V240608, 'freq': "60分钟", 'n': 5, 't1': 5, 't2': 13}]
    check_signals_acc(bars, signals_config=signals_config, height='780px', delta_days=5)  # type: ignore


if __name__ == '__main__':
    check()



import numpy as np
from collections import OrderedDict
from czsc.analyze import CZSC, BI, Direction
from czsc.utils import create_single_signal, get_sub_elements
from loguru import logger as log


def cxt_bs_V240526(c: CZSC, **kwargs) -> OrderedDict:
    """快速走势之后的减速反弹，形成第反弹买点

    参数模板："{freq}_趋势跟随_BS辅助V240526"

    **信号逻辑：**

    1. 取最近 7 笔；
    2. 如果倒数第二笔的 SNR 小于 0.7，或者倒数第二笔的力度小于前 7 笔的最大值，不考虑信号；
    3. 如果倒数第二笔是向上笔，倒数第一笔是向下笔，且倒数第一笔的力度在倒数第二笔的 10% ~ 70% 之间，形成买点；
    4. 如果倒数第二笔是向下笔，倒数第一笔是向上笔，且倒数第一笔的力度在倒数第二笔的 30% ~ 70% 之间，形成卖点。

    **信号列表：**

    - Signal('15分钟_趋势跟随_BS辅助V240526_买点_任意_任意_0')
    - Signal('15分钟_趋势跟随_BS辅助V240526_卖点_任意_任意_0')

    :param c: CZSC对象
    :param kwargs: 无
    :return: 信号识别结果
    """
    freq = c.freq.value
    k1, k2, k3 = f"{freq}_趋势跟随_BS辅助V240526".split("_")
    v1 = "其他"
    if len(c.bi_list) < 11:
        return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)

    bis = get_sub_elements(c.bi_list, di=1, n=7)
    b2, b1 = bis[-2:]
    power_price_seq = [x.power_price for x in bis]
    power_volume_seq = [x.power_volume for x in bis]
    slope_seq = [abs(x.slope) for x in bis]

    # 如果倒数第二笔的 SNR 小于 0.7，或者倒数第二笔的价格、量比最大值小于前 7 笔的最大值，不考虑信号
    if b2.SNR < 0.7 or (b2.power_price < np.max(power_price_seq)
                        and b2.power_volume < np.max(power_volume_seq)
                        and b2.slope < np.max(slope_seq)):
        return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)

    if b2.direction == Direction.Up and b1.direction == Direction.Down:
        # 买点：倒数第二笔是向上笔，倒数第一笔是向下笔，且倒数第一笔的力度在倒数第二笔的 10% ~ 70% 之间
        if 0.1 * b2.power_price < b1.power_price < 0.7 * b2.power_price:
            v1 = "买点"

    if b2.direction == Direction.Down and b1.direction == Direction.Up:
        # 卖点：倒数第二笔是向下笔，倒数第一笔是向上笔，且倒数第一笔的力度在倒数第二笔的 30% ~ 70% 之间
        if 0.2 * b2.power_price < b1.power_price < 0.7 * b2.power_price:
            v1 = "卖点"

    return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)


def check():
    from czsc.connectors import research
    from czsc.traders.base import check_signals_acc

    symbols = research.get_symbols("A股主要指数")
    bars = research.get_raw_bars(symbols[0], "5分钟", "20181101", "20210101", fq="前复权")

    signals_config = [{"name": cxt_bs_V240526, "freq": "5分钟"}]
    check_signals_acc(bars, signals_config=signals_config, height="780px", delta_days=5)  # type: ignore


if __name__ == "__main__":
    check()



import numpy as np
from collections import OrderedDict
from czsc.analyze import CZSC
from czsc.utils import create_single_signal, get_sub_elements
from loguru import logger as log


def pressure_support_V240530(c: CZSC, **kwargs) -> OrderedDict:
    """支撑压力线辅助V240530

    参数模板："{freq}_D{di}W{w}N{n}_支撑压力V240530"

    **信号逻辑：**

    对于给定K线，判断是否存在支撑压力线，判断逻辑如下：

    1. 寻找关键K线的高低点，关键K线为最近w根K线中与其他K线重叠次数最多的K线
    2. 当前K线收盘价在关键K线最高价的正负5个价位左右，认为是压力位；反之，认为是支撑位

    **信号列表：**

    - Signal('60分钟_D1W20N5_支撑压力V240530_支撑位_任意_任意_0')
    - Signal('60分钟_D1W20N5_支撑压力V240530_压力位_任意_任意_0')

    :param c: CZSC对象
    :param kwargs: 无
    :return: 信号识别结果
    """
    di = int(kwargs.get("di", 1))
    w = int(kwargs.get("w", 20))
    n = int(kwargs.get("n", 5))
    assert w > 10, "参数 w 必须大于10"

    freq = c.freq.value
    k1, k2, k3 = f"{freq}_D{di}W{w}N{n}_支撑压力V240530".split("_")
    v1 = "其他"
    if len(c.bars_raw) < w + 10:
        return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)

    bars = get_sub_elements(c.bars_raw, di=di, n=w)

    # 获取关键K线的高低点
    bar_overlap_count = {}
    for i in range(len(bars)):
        bar = bars[i]
        overlap_count = 0
        for j in range(len(bars)):
            if i == j:
                continue
            bar2 = bars[j]
            # 判断两根K线是否重叠
            if max(bar.low, bar2.low) < min(bar.high, bar2.high):
                overlap_count += 1
        bar_overlap_count[i] = overlap_count

    # 如果最大重叠次数小于总数的一半，认为没有关键K线
    if max(bar_overlap_count.values()) < 0.5 * w:
        return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)

    key_bar = bars[max(bar_overlap_count, key=bar_overlap_count.get)]
    # 获取窗口内的 unique price 列表
    prices = [y for x in c.bars_raw for y in [x.open, x.close, x.high, x.low]]
    prices = sorted(list(set(prices)))
    high_idx = prices.index(key_bar.high)
    low_idx = prices.index(key_bar.low)
    # 处理边界情况
    if high_idx < n or low_idx < n:
        return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)

    if high_idx + n >= len(prices) or low_idx + n >= len(prices):
        return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)

    # 判断压力位：当前收盘价在关键K线最高价的正负5个价位左右
    pressure_h = prices[high_idx + n]
    pressure_l = prices[high_idx - n]

    if pressure_h > bars[-1].close > pressure_l:
        v1 = "压力位"
        return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)

    # 判断支撑位：当前收盘价在关键K线最低价的正负5个价位左右
    support_h = prices[low_idx + n]
    support_l = prices[low_idx - n]
    if support_h > bars[-1].close > support_l:
        v1 = "支撑位"

    return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)


def check():
    from czsc.connectors import research
    from czsc.traders.base import check_signals_acc

    symbols = research.get_symbols("A股主要指数")
    bars = research.get_raw_bars(symbols[0], "15分钟", "20181101", "20210101", fq="前复权")

    signals_config = [{"name": pressure_support_V240530, "freq": "60分钟"}]
    check_signals_acc(bars, signals_config=signals_config, height="780px", delta_days=5)  # type: ignore


if __name__ == "__main__":
    check()



import sys

sys.path.insert(0, ".")
sys.path.insert(0, "..")
import czsc
import pandas as pd

assert czsc.WeightBacktest.version == "V240627"


def run_by_weights():
    """从持仓权重样例数据中回测"""
    dfw = pd.read_feather(r"C:\Users\zengb\Downloads\weight_example.feather")
    wb = czsc.WeightBacktest(dfw, digits=1, fee_rate=0.0002, n_jobs=1)
    # wb = czsc.WeightBacktest(dfw, digits=1, fee_rate=0.0002)
    dailys = wb.dailys
    print(wb.stats)
    print(wb.alpha_stats)
    print(wb.bench_stats)

    # 计算等权组合的超额
    df1 = dailys.groupby("date").agg({"return": "mean", "n1b": "mean"})
    df1["alpha"] = df1["return"] - df1["n1b"]

    # ------------------------------------------------------------------------------------
    # 查看绩效评价
    # ------------------------------------------------------------------------------------
    print(wb.results["绩效评价"])
    # {'开始日期': '20170103',
    #  '结束日期': '20230731',
    #  '年化': 0.093,                       # 品种等权之后的年化收益率
    #  '夏普': 1.19,                        # 品种等权之后的夏普比率
    #  '最大回撤': 0.1397,                  # 品种等权之后的最大回撤
    #  '卡玛': 0.67,
    #  '日胜率': 0.5228,                    # 品种等权之后的日胜率
    #  '年化波动率': 0.0782,
    #  '非零覆盖': 1.0,
    #  '盈亏平衡点': 0.9782,                # 品种等权之后的盈亏平衡点，这个值越小越好，正常策略的范围应该在 0.85~0.98 之间
    #  '单笔收益': 25.6,                    # 将所有品种的单笔汇总之后的平均收益，单位是 BP，即 0.01%
    #  '交易胜率': 0.3717,                  # 将所有品种的单笔汇总之后的交易胜率
    #  '持仓天数': 3.69,                    # 将所有品种的单笔汇总之后的平均持仓天数
    #  '持仓K线数': 971.66}                 # 将所有品种的单笔汇总之后的平均持仓 K 线数

    # ------------------------------------------------------------------------------------
    # 获取指定品种的回测结果
    # ------------------------------------------------------------------------------------
    symbol_res = wb.results[wb.symbols[0]]
    print(symbol_res)

    wb.report(res_path=r"C:\Users\zengb\Desktop\231005\weight_example")


if __name__ == "__main__":
    run_by_weights()



import numpy as np
from collections import OrderedDict
from czsc.analyze import CZSC, BI, Direction
from czsc.utils import create_single_signal, get_sub_elements
from loguru import logger as log


def cxt_overlap_V240526(c: CZSC, **kwargs) -> OrderedDict:
    """K线收盘价与最近9笔的顶底分型重合次数

    参数模板："{freq}_顶底重合_支撑压力V240526"

    **信号逻辑：**

    1. 取最近 15 笔；
    2. 如果当前笔是向下笔，且向下笔的底分型区间与前面的15笔中存在 t 个以上的分型区间重合，则认为是并列二买；

    **信号列表：**

    - Signal('60分钟_顶底重合_支撑压力V240526_顶重合1次_底重合0次_任意_0')
    - Signal('60分钟_顶底重合_支撑压力V240526_顶重合0次_底重合1次_任意_0')
    - Signal('60分钟_顶底重合_支撑压力V240526_顶重合0次_底重合2次_任意_0')
    - Signal('60分钟_顶底重合_支撑压力V240526_顶重合1次_底重合1次_任意_0')
    - Signal('60分钟_顶底重合_支撑压力V240526_顶重合0次_底重合0次_任意_0')
    - Signal('60分钟_顶底重合_支撑压力V240526_顶重合0次_底重合3次_任意_0')
    - Signal('60分钟_顶底重合_支撑压力V240526_顶重合2次_底重合0次_任意_0')
    - Signal('60分钟_顶底重合_支撑压力V240526_顶重合3次_底重合0次_任意_0')
    - Signal('60分钟_顶底重合_支撑压力V240526_顶重合2次_底重合1次_任意_0')
    - Signal('60分钟_顶底重合_支撑压力V240526_顶重合1次_底重合2次_任意_0')
    - Signal('60分钟_顶底重合_支撑压力V240526_顶重合4次_底重合0次_任意_0')
    - Signal('60分钟_顶底重合_支撑压力V240526_顶重合0次_底重合4次_任意_0')

    :param c: CZSC对象
    :param kwargs: 无
    :return: 信号识别结果
    """
    freq = c.freq.value
    k1, k2, k3 = f"{freq}_顶底重合_支撑压力V240526".split("_")
    v1 = "其他"
    if len(c.bi_list) < 11:
        return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)

    bis = get_sub_elements(c.bi_list, di=1, n=9)
    last_bar = c.bars_raw[-1]
    # 找出向上笔的顶分型
    fxg = [x.fx_b for x in bis if x.direction == Direction.Up]
    fxd = [x.fx_b for x in bis if x.direction == Direction.Down]

    # 与 fxg 的重合次数
    overlap_count_g = sum([1 for fx in fxg if fx.low <= last_bar.close <= fx.high])
    overlap_count_d = sum([1 for fx in fxd if fx.low <= last_bar.close <= fx.high])
    v1 = f"顶重合{overlap_count_g}次"
    v2 = f"底重合{overlap_count_d}次"
    return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1, v2=v2)


def check():
    from czsc.connectors import research
    from czsc.traders.base import check_signals_acc

    symbols = research.get_symbols("A股主要指数")
    bars = research.get_raw_bars(symbols[0], "15分钟", "20181101", "20210101", fq="前复权")

    signals_config = [{"name": cxt_overlap_V240526, "freq": "60分钟"}]
    check_signals_acc(bars, signals_config=signals_config, height="780px", delta_days=5)  # type: ignore


if __name__ == "__main__":
    check()



# 创建K线分割规则
import pandas as pd
from czsc.connectors.research import get_raw_bars


def init_kline():
    bars = get_raw_bars(symbol="000001.SH", freq="1分钟", sdt="20210903", edt="20210904", fq="后复权")
    dft = pd.DataFrame(bars)
    dft["time"] = dft["dt"].dt.strftime("%H:%M")
    dft[["time"]].to_excel("test.xlsx")

    bars = get_raw_bars(symbol="SQau9001", freq="1分钟", sdt="20210903", edt="20210904", fq="后复权")
    dft = pd.DataFrame(bars)
    dft["time"] = dft["dt"].dt.strftime("%H:%M")
    dft[["time"]].to_excel("test.xlsx")


# df = pd.read_excel(r"C:\Users\zengb\Desktop\time_split_conf_V2.xlsx")
# df['time_t'] = pd.to_datetime(df['time'], format='%H:%M')

# # for i in (3, 5, 10, 15, 20, 30, 60):
# #     df[f"{i}分钟A"] = df['time_t'].apply(lambda x: x + pd.Timedelta(minutes=i - x.minute % i))
# #     df[f"{i}分钟A"] = df[f"{i}分钟A"].dt.strftime('%H:%M').shift(1)

# # df['20分钟A'] = df['20分钟'].fillna(method='bfill')
# # for i in (2, 4, 6, 12):
# #     df[f"{i}分钟A"] = df['time_t'].apply(lambda x: x + pd.Timedelta(minutes=i - x.minute % i))
# #     df[f"{i}分钟A"] = df[f"{i}分钟A"].dt.strftime('%H:%M').shift(1)

# # df.to_excel(r"C:\Users\zengb\Desktop\time_split_conf_V2.xlsx", index=False)


# df['60分钟'] = df['60分钟'].fillna(method='bfill')
# df['120分钟'] = df['120分钟'].fillna(method='bfill')
# df.to_excel(r"C:\Users\zengb\Desktop\time_split_conf_V3.xlsx", index=False)

# df = pd.read_excel(r"C:\Users\zengb\Desktop\time_split_conf_V3.xlsx")
# df.to_feather("minutes_split.feather")

# 默认分割规则


def split_time(freq="60分钟"):
    sdt = pd.to_datetime("2021-09-03 00:00")
    res = [sdt]
    for i in range(2000):
        sdt += pd.Timedelta(minutes=1)
        res.append(sdt)
    df = pd.DataFrame(res, columns=["time"])
    df["t1"] = df["time"].dt.strftime("%H:%M")
    df1 = df.resample(freq.replace("分钟", "T"), on="time").last().reset_index()
    df1["t2"] = df1["time"].dt.strftime("%H:%M")
    dfx = pd.merge_asof(df, df1, on="time", direction="forward")
    dfx = dfx.dropna().copy()
    dfx["time"] = dfx["t1_x"]
    dfx["edt"] = dfx["t2"]
    dfx["freq"] = freq
    dfx = dfx[["time", "edt", "freq"]].drop_duplicates().sort_values("time")
    return dfx


rows = []
for freq in (
    "1分钟",
    "2分钟",
    "3分钟",
    "4分钟",
    "5分钟",
    "6分钟",
    "10分钟",
    "12分钟",
    "15分钟",
    "20分钟",
    "30分钟",
    "60分钟",
    "120分钟",
):
    rows.append(split_time(freq))

df = pd.concat(rows, ignore_index=True)
df = pd.pivot_table(df, index="time", columns="freq", values="edt", aggfunc="first").reset_index()
df["market"] = "默认"
df = df[
    [
        "market",
        "time",
        "1分钟",
        "2分钟",
        "3分钟",
        "4分钟",
        "5分钟",
        "6分钟",
        "10分钟",
        "12分钟",
        "15分钟",
        "20分钟",
        "30分钟",
        "60分钟",
        "120分钟",
    ]
]
df.to_excel(r"C:\Users\zengb\Desktop\time_split_conf_V4.xlsx", index=False)

df = pd.read_excel(r"C:\Users\zengb\Desktop\time_split_conf_V4.xlsx")
df.to_feather(r"D:\ZB\git_repo\waditu\czsc\czsc\utils\minites_split.feather")



import numpy as np
from collections import OrderedDict
from czsc.analyze import CZSC, BI, Direction
from czsc.signals.tas import update_cci_cache
from czsc.utils import create_single_signal, get_sub_elements
from loguru import logger


def cci_decision_V240620(c: CZSC, **kwargs) -> OrderedDict:
    """根据CCI指标逆势用法，判断买卖决策区域

    参数模板："{freq}_N{n}CCI_决策区域V240620"

    **信号逻辑：**

    取最近N根K线，如果最小的CCI值小于 -100，开多；如果最大的CCI值大于 100，开空。

    **信号列表：**

    - Signal('15分钟_N4CCI_决策区域V240620_开多_2次_任意_0')
    - Signal('15分钟_N4CCI_决策区域V240620_开多_1次_任意_0')
    - Signal('15分钟_N4CCI_决策区域V240620_开空_1次_任意_0')
    - Signal('15分钟_N4CCI_决策区域V240620_开空_2次_任意_0')

    :param c: CZSC对象
    :param kwargs: 无
    :return: 信号识别结果
    """
    n = int(kwargs.get("n", 2))

    freq = c.freq.value
    k1, k2, k3 = f"{freq}_N{n}CCI_决策区域V240620".split("_")
    v1 = "其他"
    cache_key = update_cci_cache(c, timeperiod=14)
    if len(c.bars_raw) < 100:
        return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)

    cci_seq = [x.cache[cache_key] for x in c.bars_raw[-n:]]
    short_cci = [x for x in cci_seq if x > 100]
    long_cci = [x for x in cci_seq if x < -100]

    v2 = "任意"
    if min(cci_seq) < -100:
        v1 = "开多"
        v2 = f"{len(long_cci)}次"
    if max(cci_seq) > 100:
        v1 = "开空"
        v2 = f"{len(short_cci)}次"
    return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1, v2=v2)


def check():
    from czsc.connectors import research
    from czsc.traders.base import check_signals_acc

    symbols = research.get_symbols("A股主要指数")
    bars = research.get_raw_bars(symbols[0], "15分钟", "20181101", "20210101", fq="前复权")

    signals_config = [{"name": cci_decision_V240620, "freq": "15分钟", "n": 4}]
    check_signals_acc(bars, signals_config=signals_config, height="780px", delta_days=5)  # type: ignore


if __name__ == "__main__":
    check()



import numpy as np
from collections import OrderedDict
from czsc.analyze import CZSC, BI, Direction
from czsc.utils import create_single_signal, get_sub_elements
from loguru import logger as log


def cxt_decision_V240526(c: CZSC, **kwargs) -> OrderedDict:
    """根据最后一根K线与最后一笔的分型区间，构建交易决策区域

    参数模板："{freq}_分型区域N{n}_决策区域V240526"

    **信号逻辑：**

    1. 取最近一根K线和最后一笔的结束分型
    2. 取100根K线，计算 unique price 序列
    3. 如果当前K线的收盘价在结束分型的 N 个 unique price 之间，认为是一个交易决策区域

    **信号列表：**

    - Signal('60分钟_分型区域N9_决策区域V240526_开多_任意_任意_0')
    - Signal('60分钟_分型区域N9_决策区域V240526_开空_任意_任意_0')

    :param c: CZSC对象
    :param kwargs: 无
    :return: 信号识别结果
    """
    n = int(kwargs.get("n", 9))

    freq = c.freq.value
    k1, k2, k3 = f"{freq}_分型区域N{n}_决策区域V240526".split("_")
    v1 = "其他"
    if len(c.bars_raw) < 120:
        return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)

    bars = get_sub_elements(c.bars_raw, di=1, n=100)
    prices = [x.close for x in bars] + [x.high for x in bars] + [x.low for x in bars] + [x.open for x in bars]
    prices = list(set(prices))

    bi = c.bi_list[-1]
    bar = c.bars_raw[-1]
    if bi.direction == Direction.Up:
        in_prices = [x for x in prices if bar.close <= x <= bi.fx_b.high]
        if len(in_prices) <= n:
            v1 = "开空"

    elif bi.direction == Direction.Down:
        in_prices = [x for x in prices if bi.fx_b.low <= x <= bar.close]
        if len(in_prices) <= n:
            v1 = "开多"

    return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)


def check():
    from czsc.connectors import research
    from czsc.traders.base import check_signals_acc

    symbols = research.get_symbols("A股主要指数")
    bars = research.get_raw_bars(symbols[0], "15分钟", "20181101", "20210101", fq="前复权")

    signals_config = [{"name": cxt_decision_V240526, "freq": "15分钟", "n": 20}]
    check_signals_acc(bars, signals_config=signals_config, height="780px", delta_days=5)  # type: ignore


if __name__ == "__main__":
    check()



import numpy as np
from collections import OrderedDict
from czsc.analyze import CZSC, BI, Direction
from czsc.utils import create_single_signal, get_sub_elements
from loguru import logger


def cxt_decision_V240614(c: CZSC, **kwargs) -> OrderedDict:
    """取最近N笔，如果当前笔向下创出新低，且累计成交量最大，开多；如果当前笔向上创出新高，且累计成交量最大，开空

    参数模板："{freq}_放量笔N{n}_决策区域V240614"

    **信号逻辑：**

    取最近N笔，如果当前笔向下创出新低，且累计成交量最大，开多；如果当前笔向上创出新高，且累计成交量最大，开空

    **信号列表：**

    - Signal('15分钟_放量笔N4_决策区域V240614_开多_任意_任意_0')
    - Signal('15分钟_放量笔N4_决策区域V240614_开空_任意_任意_0')

    :param c: CZSC对象
    :param kwargs: 无
    :return: 信号识别结果
    """
    n = int(kwargs.get("n", 4))

    freq = c.freq.value
    k1, k2, k3 = f"{freq}_放量笔N{n}_决策区域V240614".split("_")
    v1 = "其他"
    if len(c.bi_list) < n + 2 or len(c.bars_ubi) > 7:
        return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)

    bis = get_sub_elements(c.bi_list, di=1, n=n)
    bis_vol = [x.power_volume for x in bis]
    if bis[-1].power_volume != max(bis_vol):
        return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)

    if bis[-1].direction == Direction.Down and bis[-1].low != min([x.low for x in bis]):
        v1 = "开多"

    if bis[-1].direction == Direction.Up and bis[-1].high != max([x.high for x in bis]):
        v1 = "开空"

    return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)


def check():
    from czsc.connectors import research
    from czsc.traders.base import check_signals_acc

    symbols = research.get_symbols("A股主要指数")
    bars = research.get_raw_bars(symbols[0], "15分钟", "20181101", "20210101", fq="前复权")

    signals_config = [{"name": cxt_decision_V240614, "freq": "15分钟", "n": 4}]
    check_signals_acc(bars, signals_config=signals_config, height="780px", delta_days=5)  # type: ignore


if __name__ == "__main__":
    check()



from collections import OrderedDict

import numpy as np

from copy import deepcopy
from czsc.analyze import CZSC
from czsc.utils import create_single_signal, get_sub_elements


def bar_td9_V240616(c: CZSC, **kwargs) -> OrderedDict:
    """神奇九转计数

    参数模板："{freq}_神奇九转N{n}_BS辅助V240616"

    **信号逻辑：**

    1. 当前收盘价大于前4根K线的收盘价，+1，否则-1
    2. 如果最后一根K线为1，且连续值计数大于等于N，卖点；如果最后一根K线为-1，且连续值计数小于等于-N，买点

    **信号列表：**

    - Signal('60分钟_神奇九转N9_BS辅助V240616_买点_9转_任意_0')
    - Signal('60分钟_神奇九转N9_BS辅助V240616_卖点_9转_任意_0')

    :param c: CZSC对象
    :param kwargs:

        - n: int, default 9, 连续转折次数

    :return: 信号识别结果
    """
    n = int(kwargs.get("n", 9))

    freq = c.freq.value
    k1, k2, k3 = f"{freq}_神奇九转N{n}_BS辅助V240616".split("_")
    v1 = "其他"

    # 更新缓存
    cache_key = "bar_td9_V240616"
    for i, bar in enumerate(c.bars_raw):
        if i < 4 or hasattr(bar.cache, cache_key):
            continue

        if bar.close > c.bars_raw[i - 4].close:
            bar.cache[cache_key] = 1
        elif bar.close < c.bars_raw[i - 4].close:
            bar.cache[cache_key] = -1
        else:
            bar.cache[cache_key] = 0

    if len(c.bars_raw) < 30 + n:
        return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)

    v2 = "任意"
    bars = get_sub_elements(c.bars_raw, di=1, n=n * 2)
    if bars[-1].cache[cache_key] == 1:
        count = 0
        for bar in bars[::-1]:
            if bar.cache[cache_key] != 1:
                break
            count += 1
        if count >= n:
            v1 = "卖点"
            v2 = f"{count}转"

    elif bars[-1].cache[cache_key] == -1:
        count = 0
        for bar in bars[::-1]:
            if bar.cache[cache_key] != -1:
                break
            count += 1
        if count >= n:
            v1 = "买点"
            v2 = f"{count}转"

    return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1, v2=v2)


def check():
    from czsc.connectors import research
    from czsc.traders.base import check_signals_acc

    symbols = research.get_symbols("A股主要指数")
    bars = research.get_raw_bars(symbols[0], "15分钟", "20181101", "20210101", fq="前复权")

    signals_config = [
        {"name": bar_td9_V240616, "freq": "60分钟"},
    ]
    check_signals_acc(bars, signals_config=signals_config, height="780px", delta_days=5)  # type: ignore


if __name__ == "__main__":
    check()



import numpy as np
from collections import OrderedDict
from czsc.analyze import CZSC, Direction
from czsc.signals.tas import update_macd_cache
from czsc.utils import create_single_signal


def tas_dif_zero_V240614(c: CZSC, **kwargs) -> OrderedDict:
    """DIFF 远离零轴后靠近零轴，形成买卖点

    参数模板："{freq}_DIF靠近零轴W{w}T{t}_BS辅助V240614"

    **信号逻辑：**

    买点的定位以DIF为主，要求如下。

    1，取最近 w 根K线，获取 diffs 序列
    2. 如果所有 diff 都大于 0，且最近一个 diff 在 0.5倍标准差范围内，且最大值大于均值加标准差，则认为是买点

    **信号列表：**

    - Signal('60分钟_DIF靠近零轴W20T50_BS辅助V240614_卖点_任意_任意_0')
    - Signal('60分钟_DIF靠近零轴W20T50_BS辅助V240614_买点_任意_任意_0')

    :param c: CZSC对象
    :param kwargs: 无

        - t: DIF波动率的倍数，除以100，默认为50

    :return: 信号识别结果
    """
    w = int(kwargs.get("w", 20))  # K线数量
    t = int(kwargs.get("t", 50))  # 波动率的倍数，除以100

    freq = c.freq.value
    k1, k2, k3 = f"{freq}_DIF靠近零轴W{w}T{t}_BS辅助V240614".split("_")
    v1 = "其他"
    key = update_macd_cache(c)
    if len(c.bars_raw) < 110:
        return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)

    bars = c.bars_raw[-w:]
    diffs = [x.cache[key]["dif"] for x in bars]
    delta = np.std(diffs) * t / 100
    max_diff = max(diffs)
    min_diff = min(diffs)
    abs_mean_diff = abs(np.mean(diffs))
    std_diff = np.std(diffs)

    if all(x > 0 for x in diffs) and delta > diffs[-1] > -delta and max_diff > abs_mean_diff + std_diff:
        v1 = "买点"

    if all(x < 0 for x in diffs) and -delta < diffs[-1] < delta and min_diff < -(abs_mean_diff + std_diff):
        v1 = "卖点"

    return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)


def check():
    from czsc.connectors import research
    from czsc.traders.base import check_signals_acc

    symbols = research.get_symbols("A股主要指数")
    bars = research.get_raw_bars(symbols[0], "15分钟", "20181101", "20210101", fq="前复权")

    signals_config = [{"name": tas_dif_zero_V240614, "freq": "60分钟", "t": 50}]
    check_signals_acc(bars, signals_config=signals_config, height="780px", delta_days=5)  # type: ignore


if __name__ == "__main__":
    check()



import numpy as np
from collections import OrderedDict
from czsc.analyze import CZSC, BI, Direction
from czsc.utils import create_single_signal, get_sub_elements
from loguru import logger


def cxt_decision_V240612(c: CZSC, **kwargs) -> OrderedDict:
    """以最近W根K线的高低点附近N个价位作为决策区域

    参数模板："{freq}_W{w}N{n}高低点_决策区域V240612"

    **信号逻辑：**

    1. 取最近W根K线的高点和低点，分别找出第N个价位，作为决策区域的上下界；
    2. 当前K线的最高价超过决策区域的上界，开空；
    3. 当前K线的最低价低于决策区域的下界，开多；

    **信号列表：**

    - Signal('15分钟_W10N5高低点_决策区域V240612_开多_任意_任意_0')
    - Signal('15分钟_W10N5高低点_决策区域V240612_开空_任意_任意_0')

    :param c: CZSC对象
    :param kwargs: 无
    :return: 信号识别结果
    """
    w = int(kwargs.get("w", 10))
    n = int(kwargs.get("n", 9))

    freq = c.freq.value
    k1, k2, k3 = f"{freq}_W{w}N{n}高低点_决策区域V240612".split("_")
    v1 = "其他"
    if len(c.bars_raw) < 120:
        return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)

    bars = get_sub_elements(c.bars_raw, di=1, n=100)
    prices = [x.close for x in bars] + [x.high for x in bars] + [x.low for x in bars] + [x.open for x in bars]
    prices = list(set(prices))

    w_bars = get_sub_elements(c.bars_raw, di=1, n=w)

    max_high = max([x.high for x in w_bars])
    min_low = min([x.low for x in w_bars])
    last_bar = c.bars_raw[-1]

    # 低点上方的第N个价位
    min_low_upper = sorted([x for x in prices if x >= min_low])
    low_range = min_low_upper[n] if len(min_low_upper) > n else min_low_upper[-1]

    # 高点下方的第N个价位
    max_high_lower = sorted([x for x in prices if x <= max_high], reverse=True)
    high_range = max_high_lower[n] if len(max_high_lower) > n else max_high_lower[-1]

    if last_bar.close < low_range and last_bar.low != min_low:
        v1 = "开多"

    if last_bar.close > high_range and last_bar.high != max_high:
        v1 = "开空"

    return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)


def check():
    from czsc.connectors import research
    from czsc.traders.base import check_signals_acc

    symbols = research.get_symbols("A股主要指数")
    bars = research.get_raw_bars(symbols[0], "15分钟", "20181101", "20210101", fq="前复权")

    signals_config = [{"name": cxt_decision_V240612, "freq": "15分钟", "n": 5}]
    check_signals_acc(bars, signals_config=signals_config, height="780px", delta_days=5)  # type: ignore


if __name__ == "__main__":
    check()



import os
import sys
import dotenv

os.chdir(r"A:\ZB\git_repo\waditu\czsc\examples\test_offline")
sys.path.insert(0, r"A:\ZB\git_repo\waditu\czsc")
dotenv.load_dotenv(dotenv.find_dotenv(raise_error_if_not_found=True), override=True)
from czsc.fsa import BiTable

bi_table = BiTable(app_token="ZkWzbY7xjaicgkslpdUc1tQLnk8")

print(
    bi_table.create_table(
        name="test2",
        default_view_name="视图1",
        fields=[{"field_name": "text", "type": 1}, {"field_name": "text1", "type": 2}],
    )
)
print(bi_table.batch_create_table(["tb1", "tb2", "tb3"]))
print(bi_table.delete_table("tblNW6s4ldB3dhCG"))
print(bi_table.batch_delete_table(["tbly1sheIukE622s", "tblNK7XcwQDLdomG"]))
print(bi_table.patch_table("tblzRoIFq3URau2V", "name_patch"))
print(bi_table.list_tables(page_token="tbltFoOtwXCuhtj9"))

print(bi_table.table_record_get(table_id="tblfSD2jLnUMi4sE", record_id="recwyiPrHM"))
print(bi_table.table_record_search(table_id="tblfSD2jLnUMi4sE"))
print(bi_table.table_record_create(table_id="tblfSD2jLnUMi4sE", fields={"文本": "多行文本内容", "日期": 1674206443000}))

print(
    bi_table.table_record_update(
        table_id="tblfSD2jLnUMi4sE",
        record_id="recuhm4cMqw7fn",
        fields={
            "文本": "多行文本内容修改",
        },
    )
)
print(bi_table.table_record_delete(table_id="tblfSD2jLnUMi4sE", record_id="recwyiPrHM"))

print(
    bi_table.table_record_batch_create(
        table_id="tblfSD2jLnUMi4sE",
        fields=[{"文本": "多行文本内容1"}, {"文本": "多行文本内容2"}, {"文本": "多行文本内容3", "日期": 1674206443000}],
    )
)

print(
    bi_table.table_record_batch_update(
        table_id="tblfSD2jLnUMi4sE",
        records=[
            {"record_id": "recuhm9s2axNVM", "fields": {"文本": "批量修改内容1"}},
            {"record_id": "recuhm9s2aZ3At", "fields": {"文本": "批量修改内容2"}},
        ],
    )
)

print(bi_table.table_record_batch_delete(table_id="tblfSD2jLnUMi4sE", record_ids=["recuhm9s2axNVM", "recuhm9s2aZ3At"]))

print(bi_table.table_view_list(table_id="tblfSD2jLnUMi4sE"))
print(bi_table.table_field_list(table_id="tblfSD2jLnUMi4sE", view_id="vewo15a8k6"))

print(bi_table.table_view_patch(table_id="tblfSD2jLnUMi4sE", view_id="vewo15a8k6", infos={"view_name": "修改的视图名"}))

print(bi_table.table_view_get(table_id="tblfSD2jLnUMi4sE", view_id="vewo15a8k6"))

print(bi_table.table_view_create(table_id="tblfSD2jLnUMi4sE", view_name="测试添加视图", view_type="grid"))
print(bi_table.table_view_delete(table_id="tblfSD2jLnUMi4sE", view_id="vew4hoglsH"))

print(
    bi_table.table_field_create(
        table_id="tblfSD2jLnUMi4sE", field_name="测试添加2", type=1, description={"text": "测试的"}
    )
)
print(
    bi_table.table_field_update(
        table_id="tblfSD2jLnUMi4sE",
        field_id="fldEbAErbT",
        field_name="测试添加2修改",
        type=11,
        description={"text": "测试的 并修改"},
        property={"multiple": True},
    )
)

print(bi_table.table_field_delete(table_id="tblfSD2jLnUMi4sE", field_id="fldEbAErbT"))

print(bi_table.table_form_list(table_id="tblfSD2jLnUMi4sE", form_id="vewafwFMhM"))
print(
    bi_table.table_form_patch(
        table_id="tblfSD2jLnUMi4sE", form_id="vewafwFMhM", name="测试修改的名字", description="测试修改的备注"
    )
)
print(bi_table.table_form_get(table_id="tblfSD2jLnUMi4sE", form_id="vewafwFMhM"))
print(
    bi_table.table_form_patch(
        table_id="tblfSD2jLnUMi4sE",
        form_id="vewafwFMhM",
        field_id="fld7mfWuZ2",
        title="修改的title",
        description="api修改的",
    )
)

print(bi_table.table_copy(name="测试添加的"))
print(bi_table.table_create(name="测试添加的"))
print(bi_table.table_get(app_token="YuSZbIPLlaPenUsOzbfcL25Sn4g"))
print(bi_table.table_update(app_token="YuSZbIPLlaPenUsOzbfcL25Sn4g", name="修改成的新名字"))



import numpy as np
from collections import OrderedDict
from czsc.analyze import CZSC, BI, Direction
from czsc.utils import create_single_signal, get_sub_elements
from loguru import logger as log


def cxt_overlap_V240612(c: CZSC, **kwargs) -> OrderedDict:
    """顺畅笔的顶底分型构建支撑压力位

    参数模板："{freq}_SNR顺畅N{n}_支撑压力V240612"

    **信号逻辑：**

    前面N笔中走势最顺畅的笔，顶底分型是重要支撑压力位，可以作为决策点。

    1. 取最近 N 笔，找出SNR最大的笔 B1；
    2. 如果当前笔是向下笔，且向下笔的底分型区间与B1的顶/底分型区间有重合，那么认为当前位置是支撑位；

    **信号列表：**

    - Signal('60分钟_SNR顺畅N9_支撑压力V240612_支撑_顺畅笔底分型_任意_0')
    - Signal('60分钟_SNR顺畅N9_支撑压力V240612_压力_顺畅笔底分型_任意_0')
    - Signal('60分钟_SNR顺畅N9_支撑压力V240612_支撑_顺畅笔顶分型_任意_0')
    - Signal('60分钟_SNR顺畅N9_支撑压力V240612_压力_顺畅笔顶分型_任意_0')

    :param c: CZSC对象
    :param kwargs: 无
    :return: 信号识别结果
    """
    n = int(kwargs.get("n", 7))
    freq = c.freq.value
    k1, k2, k3 = f"{freq}_SNR顺畅N{n}_支撑压力V240612".split("_")
    v1 = "其他"
    if len(c.bi_list) < n + 2 or len(c.bars_ubi) > 7:
        return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)

    bis = get_sub_elements(c.bi_list, di=3, n=n)
    bis = [x for x in bis if len(x.raw_bars) >= 9]
    if len(bis) == 0:
        return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)

    max_snr_bi = max(bis, key=lambda x: x.SNR)
    if max_snr_bi.SNR < 0.7:
        return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)

    if max_snr_bi.direction == Direction.Down:
        fxg = max_snr_bi.fx_a
        fxd = max_snr_bi.fx_b
    else:
        fxg = max_snr_bi.fx_b
        fxd = max_snr_bi.fx_a

    def is_price_overlap(h1, l1, h2, l2):
        """判断两个价格区间是否有重合"""
        return True if max(l1, l2) < min(h1, h2) else False

    last_bi = c.bi_list[-1]
    v2 = "任意"
    if last_bi.direction == Direction.Down:
        if is_price_overlap(fxg.high, fxg.low, last_bi.fx_b.high, last_bi.fx_b.low):
            v1 = "支撑"
            v2 = "顺畅笔顶分型"
        if is_price_overlap(fxd.high, fxd.low, last_bi.fx_b.high, last_bi.fx_b.low):
            v1 = "支撑"
            v2 = "顺畅笔底分型"

    if last_bi.direction == Direction.Up:
        if is_price_overlap(fxg.high, fxg.low, last_bi.fx_b.high, last_bi.fx_b.low):
            v1 = "压力"
            v2 = "顺畅笔顶分型"

        if is_price_overlap(fxd.high, fxd.low, last_bi.fx_b.high, last_bi.fx_b.low):
            v1 = "压力"
            v2 = "顺畅笔底分型"

    return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1, v2=v2)


def check():
    from czsc.connectors import research
    from czsc.traders.base import check_signals_acc

    symbols = research.get_symbols("A股主要指数")
    bars = research.get_raw_bars(symbols[0], "15分钟", "20181101", "20210101", fq="前复权")

    signals_config = [{"name": cxt_overlap_V240612, "freq": "60分钟"}]
    check_signals_acc(bars, signals_config=signals_config, height="780px", delta_days=5)  # type: ignore


if __name__ == "__main__":
    check()



from collections import OrderedDict
from czsc.analyze import CZSC
from czsc.utils import create_single_signal


def bar_classify_V240606(c: CZSC, **kwargs) -> OrderedDict:
    """单根K线收盘位置分类

    飞书文档：https://s0cqcxuy3p.feishu.cn/wiki/P79fwVE19i7vw4keDuac4tFRnMf

    参数模板："{freq}_D{di}收盘位置_分类V240606"

    **信号逻辑：**

    1. 高收盘蜡烛是指收盘价在蜡烛范围的上三分之一内的蜡烛。
    2. 中间收盘价是指收盘价在蜡烛范围的中间三分之一以内。
    3. 中间收盘价是指收盘价在蜡烛范围的中间三分之一以内。

    **信号列表：**

    - Signal('60分钟_D1收盘位置_分类V240606_低位_任意_任意_0')
    - Signal('60分钟_D1收盘位置_分类V240606_中间_任意_任意_0')
    - Signal('60分钟_D1收盘位置_分类V240606_高位_任意_任意_0')

    :param c: CZSC对象
    :param kwargs:

        - di: int, default 1, 周期偏移量

    :return: 信号识别结果
    """
    di = int(kwargs.get("di", 1))
    assert di > 0, "参数 di 必须大于 0"

    freq = c.freq.value
    k1, k2, k3 = f"{freq}_D{di}收盘位置_分类V240606".split("_")
    v1 = "其他"
    if len(c.bars_raw) < 7 + di:
        return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)

    bar = c.bars_raw[-di]
    close, high, low = bar.close, bar.high, bar.low
    gap_unit = (high - low) / 3
    if close > (high - gap_unit):
        v1 = "高位"
    elif close < (low + gap_unit):
        v1 = "低位"
    else:
        v1 = "中间"

    return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)


def check():
    from czsc.connectors import research
    from czsc.traders.base import check_signals_acc

    symbols = research.get_symbols("A股主要指数")
    bars = research.get_raw_bars(symbols[0], "15分钟", "20181101", "20210101", fq="前复权")

    signals_config = [
        {"name": bar_classify_V240606, "freq": "60分钟"},
        # {"name": bar_plr_V240427, "freq": "60分钟", "m": "空头"},
    ]
    check_signals_acc(bars, signals_config=signals_config, height="780px", delta_days=5)  # type: ignore


if __name__ == "__main__":
    check()



from collections import OrderedDict

import numpy as np

from copy import deepcopy
from czsc.analyze import CZSC
from czsc.utils import create_single_signal, get_sub_elements


def bar_decision_V240608(c: CZSC, **kwargs) -> OrderedDict:
    """W窗口内最近N根K线放量后进行决策，如果是多头放量，开空，如果是空头放量，开多

    参数模板："{freq}_W{w}N{n}Q{q}放量_决策区域V240608"

    **信号逻辑：**

    1. 看多：第二根K线收盘价在第一根K线的最高价上方
    2. 看空：第二根K线收盘价在第一根K线的最低价下方
    3. 中性：其他情况

    **信号列表：**

    - Signal('60分钟_W300N20Q70放量_决策区域V240608_看空_任意_任意_0')
    - Signal('60分钟_W300N20Q70放量_决策区域V240608_放量_任意_任意_0')
    - Signal('60分钟_W300N20Q70放量_决策区域V240608_看多_任意_任意_0')

    :param c: CZSC对象
    :param kwargs:

        - w: int, default 300, 窗口大小
        - n: int, default 20, 最近N根K线
        - q: int, default 80, 分位数，取值范围 0-100，表示取最近 w 根K线的成交量的 q 分位数

    :return: 信号识别结果
    """
    w = int(kwargs.get("w", 300))
    n = int(kwargs.get("n", 10))
    q = int(kwargs.get("q", 80))  # 分位数，取值范围 0-100，表示取最近 w 根K线的成交量的 q 分位数
    assert w > n > 3, "参数 w 必须大于 n，且 n 必须大于 0"

    freq = c.freq.value
    k1, k2, k3 = f"{freq}_W{w}N{n}Q{q}放量_决策区域V240608".split("_")
    v1 = "其他"
    if len(c.bars_raw) < w + n:
        return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)

    w_bars = get_sub_elements(c.bars_raw, di=1, n=w)
    n_bars = get_sub_elements(c.bars_raw, di=1, n=n)
    n_bars = deepcopy(n_bars)
    n_diff = n_bars[-1].close - n_bars[0].open

    # 找出 n_bars 中成交量最大的3根K线
    n_bars.sort(key=lambda x: x.vol, reverse=True)
    n_bars = n_bars[:3]

    # 计算 w_bars 中成交量的 q 分位数
    qth = np.quantile([x.vol for x in w_bars], q / 100)
    vol_match = all([x.vol > qth for x in n_bars])
    if vol_match and n_diff > 0:
        v1 = "看空"
    if vol_match and n_diff < 0:
        v1 = "看多"
    return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)


def check():
    from czsc.connectors import research
    from czsc.traders.base import check_signals_acc

    symbols = research.get_symbols("A股主要指数")
    bars = research.get_raw_bars(symbols[0], "15分钟", "20181101", "20210101", fq="前复权")

    signals_config = [
        {"name": bar_decision_V240608, "freq": "60分钟", "q": 80},
    ]
    check_signals_acc(bars, signals_config=signals_config, height="780px", delta_days=5)  # type: ignore


if __name__ == "__main__":
    check()
